@@ -2,7 +2,6 @@ package sysregistriesv2
 
 import (
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"testing"
@@ -70,8 +69,7 @@ func TestParseLocation(t *testing.T) {
 
 	// invalid locations
 	_, err = parseLocation("https://example.com")
-	assert.NotNil(t, err)
-	assert.Contains(t, err.Error(), "invalid location 'https://example.com': URI schemes are not supported")
+	assert.ErrorContains(t, err, "invalid location 'https://example.com': URI schemes are not supported")
 
 	_, err = parseLocation("john.doe@example.com")
 	assert.Nil(t, err)
@@ -211,6 +209,11 @@ func TestRefMatchingPrefix(t *testing.T) {
 		{"example.com/foo@sha256:aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", "example.com/foo@sha256:aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
 			len("example.com/foo@sha256:aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")},
 		{"example.com/foo@sha256:bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb", "example.com/foo@sha256:aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", -1},
+		// Prefix is invalid, but we shouldn’t crash.
+		// (Note that this is necessary only because loadConfigFile doesn’t reject single-character values outright,
+		// which it, in principle, could; a valid prefix/location must start with a host name, and host names
+		// that could ever match anything contain either a dot or a port number, due to docker.io normalization rules.)
+		{"example.com/foo", "*", -1},
 	} {
 		prefixLen := refMatchingPrefix(c.ref, c.prefix)
 		assert.Equal(t, c.expected, prefixLen, fmt.Sprintf("%s vs. %s", c.ref, c.prefix))
@@ -221,9 +224,7 @@ func TestNewConfigWrapper(t *testing.T) {
 	const nondefaultPath = "/this/is/not/the/default/registries.conf"
 	const variableReference = "$HOME"
 	const rootPrefix = "/root/prefix"
-	tempHome, err := ioutil.TempDir("", "tempHome")
-	require.NoError(t, err)
-	defer os.RemoveAll(tempHome)
+	tempHome := t.TempDir()
 	var userRegistriesFile = filepath.FromSlash(".config/containers/registries.conf")
 	userRegistriesFilePath := filepath.Join(tempHome, userRegistriesFile)
 
@@ -429,7 +430,7 @@ func TestInvalidV2Configs(t *testing.T) {
 		_, err := GetRegistries(&types.SystemContext{SystemRegistriesConfPath: c.path})
 		assert.Error(t, err, c.path)
 		if c.errorSubstring != "" {
-			assert.Contains(t, err.Error(), c.errorSubstring, c.path)
+			assert.ErrorContains(t, err, c.errorSubstring, c.path)
 		}
 	}
 }
@@ -478,17 +479,16 @@ func TestMixingV1andV2(t *testing.T) {
 		SystemRegistriesConfPath:    "testdata/mixing-v1-v2.conf",
 		SystemRegistriesConfDirPath: "testdata/this-does-not-exist",
 	})
-	require.Error(t, err)
-	assert.Contains(t, err.Error(), "mixing sysregistry v1/v2 is not supported")
+	assert.ErrorContains(t, err, "mixing sysregistry v1/v2 is not supported")
 }
 
 func TestConfigCache(t *testing.T) {
-	configFile, err := ioutil.TempFile("", "sysregistriesv2-test")
+	configFile, err := os.CreateTemp("", "sysregistriesv2-test")
 	require.NoError(t, err)
 	defer os.Remove(configFile.Name())
 	defer configFile.Close()
 
-	err = ioutil.WriteFile(configFile.Name(), []byte(`
+	err = os.WriteFile(configFile.Name(), []byte(`
 [[registry]]
 location = "registry.com"
 
@@ -523,7 +523,7 @@ insecure = true`), 0600)
 
 	// empty the config, but use the same SystemContext to show that the
 	// previously specified registries are in the cache
-	err = ioutil.WriteFile(configFile.Name(), []byte{}, 0600)
+	err = os.WriteFile(configFile.Name(), []byte{}, 0600)
 	require.NoError(t, err)
 	registries, err = GetRegistries(ctx)
 	assert.Nil(t, err)
@@ -670,6 +670,122 @@ func TestPullSourcesFromReference(t *testing.T) {
 	assert.Equal(t, 1, len(pullSources))
 }
 
+func TestPullSourcesMirrorFromReference(t *testing.T) {
+	sys := &types.SystemContext{
+		SystemRegistriesConfPath:    "testdata/pull-sources-mirror-reference.conf",
+		SystemRegistriesConfDirPath: "testdata/this-does-not-exist",
+	}
+	registries, err := GetRegistries(sys)
+	require.NoError(t, err)
+	assert.Equal(t, 7, len(registries))
+
+	digest := "@sha256:aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
+	tag := ":aaa"
+	for _, tc := range []struct {
+		registry      string
+		digestSources []string
+		tagSources    []string
+	}{
+		// Registry A has mirrors allow any kind of pull
+		{
+			"registry-a.com/foo/image",
+			[]string{"mirror-1.registry-a.com", "mirror-2.registry-a.com", "registry-a.com/bar"},
+			[]string{"mirror-1.registry-a.com", "mirror-2.registry-a.com", "registry-a.com/bar"},
+		},
+		// Registry B has mirrors allow digests pull only
+		{
+			"registry-b.com/foo/image",
+			[]string{"mirror-1.registry-b.com", "mirror-2.registry-b.com", "registry-b.com/bar"},
+			[]string{"registry-b.com/bar"},
+		},
+		// Registry C has a mirror allows digest pull only and a mirror allows any kind of pull
+		{
+			"registry-c.com/foo/image",
+			[]string{"mirror-1.registry-c.com", "mirror-2.registry-c.com", "registry-c.com/bar"},
+			[]string{"mirror-1.registry-c.com", "registry-c.com/bar"},
+		},
+		// Registry D set digest-only for registry level, allows only digest pulls
+		// Registry D has no digest-only set for mirrors table
+		{
+			"registry-d.com/foo/image",
+			[]string{"mirror-1.registry-d.com", "mirror-2.registry-d.com", "registry-d.com/bar"},
+			[]string{"registry-d.com/bar"},
+		},
+		// Registry E has mirrors only allows tag pull
+		{
+			"registry-e.com/foo/image",
+			[]string{"registry-e.com/bar"},
+			[]string{"mirror-1.registry-e.com", "mirror-2.registry-e.com", "registry-e.com/bar"},
+		},
+		// Registry F has one tag only mirror does not allow digest pull
+		{
+			"registry-f.com/foo/image",
+			[]string{"mirror-1.registry-f.com", "registry-f.com/bar"},
+			[]string{"mirror-1.registry-f.com", "mirror-2.registry-f.com", "registry-f.com/bar"},
+		},
+		// Registry G has one digest-only pull and one tag only pull
+		{
+			"registry-g.com/foo/image",
+			[]string{"mirror-1.registry-g.com", "mirror-3.registry-g.com", "mirror-4.registry-g.com", "registry-g.com/bar"},
+			[]string{"mirror-2.registry-g.com", "mirror-3.registry-g.com", "mirror-4.registry-g.com", "registry-g.com/bar"},
+		},
+	} {
+		// Digest
+		digestedRef := toNamedRef(t, tc.registry+digest)
+		registry, err := FindRegistry(sys, digestedRef.Name())
+		require.NoError(t, err)
+		require.NotNil(t, registry)
+		pullSource, err := registry.PullSourcesFromReference(digestedRef)
+		require.NoError(t, err)
+		for i, s := range tc.digestSources {
+			assert.Equal(t, s, pullSource[i].Endpoint.Location)
+		}
+		// Tag
+		taggedRef := toNamedRef(t, tc.registry+tag)
+		registry, err = FindRegistry(sys, taggedRef.Name())
+		require.NoError(t, err)
+		require.NotNil(t, registry)
+		pullSource, err = registry.PullSourcesFromReference(taggedRef)
+		require.NoError(t, err)
+		for i, s := range tc.tagSources {
+			assert.Equal(t, s, pullSource[i].Endpoint.Location)
+		}
+	}
+}
+
+func TestInvalidMirrorConfig(t *testing.T) {
+	for _, tc := range []struct {
+		sys       *types.SystemContext
+		expectErr string
+	}{
+		{
+			sys: &types.SystemContext{
+				SystemRegistriesConfPath:    "testdata/invalid-config-level-mirror.conf",
+				SystemRegistriesConfDirPath: "testdata/this-does-not-exist",
+			},
+			expectErr: fmt.Sprintf("pull-from-mirror must not be set for a non-mirror registry %q", "registry-a.com/foo"),
+		},
+		{
+			sys: &types.SystemContext{
+				SystemRegistriesConfPath:    "testdata/invalid-conflict-mirror.conf",
+				SystemRegistriesConfDirPath: "testdata/this-does-not-exist",
+			},
+			expectErr: fmt.Sprintf("cannot set mirror usage mirror-by-digest-only for the registry (%q) and pull-from-mirror for per-mirror (%q) at the same time", "registry-a.com/foo", "mirror-1.registry-a.com"),
+		},
+		{
+			sys: &types.SystemContext{
+				SystemRegistriesConfPath:    "testdata/invalid-value-mirror.conf",
+				SystemRegistriesConfDirPath: "testdata/this-does-not-exist",
+			},
+			expectErr: fmt.Sprintf("unsupported pull-from-mirror value %q for mirror %q", "notvalid", "mirror-1.registry-a.com"),
+		},
+	} {
+		_, err := GetRegistries(tc.sys)
+		assert.ErrorContains(t, err, tc.expectErr)
+	}
+
+}
+
 func TestTryUpdatingCache(t *testing.T) {
 	ctx := &types.SystemContext{
 		SystemRegistriesConfPath:    "testdata/try-update-cache-valid.conf",
